@@ -14,7 +14,6 @@
 import itertools
 
 from keystoneauth1 import loading
-from oslo_config import cfg
 
 import aodh.api
 import aodh.api.controllers.v2.alarms
@@ -25,7 +24,7 @@ import aodh.evaluator.gnocchi
 import aodh.event
 import aodh.keystone_client
 import aodh.notifier.rest
-import aodh.rpc
+import aodh.notifier.zaqar
 import aodh.service
 import aodh.storage
 
@@ -36,37 +35,23 @@ def list_opts():
          itertools.chain(
              aodh.evaluator.OPTS,
              aodh.evaluator.event.OPTS,
-             aodh.evaluator.gnocchi.OPTS,
-             aodh.event.OPTS,
-             aodh.notifier.OPTS,
+             aodh.evaluator.threshold.OPTS,
              aodh.notifier.rest.OPTS,
              aodh.queue.OPTS,
-             aodh.rpc.OPTS,
-             aodh.service.OPTS,
-             aodh.api.controllers.v2.alarms.ALARM_API_OPTS)),
+             aodh.service.OPTS)),
         ('api',
          itertools.chain(
              aodh.api.OPTS,
-             [
-                 cfg.StrOpt(
-                     'paste_config',
-                     deprecated_name='api_paste_config',
-                     deprecated_group='DEFAULT',
-                     default="api_paste.ini",
-                     help="Configuration file for WSGI definition of API."),
-                 cfg.IntOpt(
-                     'workers', default=1,
-                     deprecated_name='api_workers',
-                     deprecated_group='DEFAULT',
-                     min=1,
-                     help='Number of workers for aodh API server.'),
-                 cfg.BoolOpt('pecan_debug',
-                             default=False,
-                             help='Toggle Pecan Debug Middleware.'),
-             ])),
+             aodh.api.controllers.v2.alarms.ALARM_API_OPTS)),
         ('coordination', aodh.coordination.OPTS),
         ('database', aodh.storage.OPTS),
+        ('evaluator', aodh.service.EVALUATOR_OPTS),
+        ('listener', itertools.chain(aodh.service.LISTENER_OPTS,
+                                     aodh.event.OPTS)),
+        ('notifier', aodh.service.NOTIFIER_OPTS),
         ('service_credentials', aodh.keystone_client.OPTS),
+        ('service_types', aodh.notifier.zaqar.SERVICE_OPTS),
+        ('notifier', aodh.notifier.OPTS),
     ]
 
 
@@ -74,8 +59,6 @@ def list_keystoneauth_opts():
     # NOTE(sileht): the configuration file contains only the options
     # for the password plugin that handles keystone v2 and v3 API
     # with discovery. But other options are possible.
-    # Also, the default loaded plugin is password-aodh-legacy for
-    # backward compatibily
     return [('service_credentials', (
             loading.get_auth_common_conf_options() +
             loading.get_auth_plugin_conf_options('password')))]
